# data_combo_row.py
#
# Copyright 2024 Hari Rana (TheEvilSkeleton)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0-or-later


import os
from glob import glob
from typing import Any, cast

from gi.repository import GObject, Adw, Gtk, Gio

from refine.widgets.reset_button import RefineResetButton
from refine.utils import (
    RefineSettings,
    create_schema_source,
    set_up_separator_revealer,
    get_system_data_dirs,
    get_user_data_dir,
)

import logging


@Gtk.Template(resource_path="/page/tesk/Refine/widgets/data-combo-row.ui")
class RefineDataComboRow(Adw.ComboRow):
    """
    An `Adw.ComboRow` implementing the properties needed to
    work with `Gio.Settings`.
    """

    __gtype_name__ = "RefineDataComboRow"

    revert_button = Gtk.Template.Child()  # type: ignore [misc]
    revealer = Gtk.Template.Child()  # type: ignore [misc]

    is_valid_setting = GObject.Property(type=bool, default=True)
    settings = GObject.Property(type=Gio.Settings)
    restart_required = GObject.Property(type=bool, default=False)
    lookup_directory = GObject.Property(type=str, default="")
    is_default = GObject.Property(type=bool, default=True)
    schema = GObject.Property(type=Gio.SettingsSchema)

    def __init__(self, **kwargs: Any) -> None:
        super().__init__(**kwargs)

        self.bind_property("is-valid-setting", self, "sensitive")

        self.connect("notify::key", self.__key_set)

    def __key_set(self, *args) -> None:
        self.revert_button.connect("clicked", self.__reset_button_clicked)
        self.bind_property(
            "is-default",
            self.revert_button,
            "sensitive",
            GObject.BindingFlags.SYNC_CREATE,
        )
        set_up_separator_revealer(self.revealer, self.revert_button, "sensitive")
        self.connect("inconsistent", self.__inconsistent_set)
        self.settings.connect("changed", self.__settings_changed)

    def __settings_changed(self, *args: Any) -> None:
        if not (model := cast(Gtk.StringList, self.get_model())):
            return

        row_string = model.get_string(self.get_selected())
        settings_string = self.settings.get_string(self.key)

        if row_string != settings_string:
            self.emit("inconsistent")

    def __inconsistent_set(self, *args: Any) -> None:
        self.handler_block_by_func(self.__selected)

        if not (model := cast(Gtk.StringList, self.get_model())):
            return

        row_string = self.settings.get_string(self.key)

        self.__set_selected(model, row_string)
        self.__set_default(cast(str, self.key), model)

        self.handler_unblock_by_func(self.__selected)

    def __reset_button_clicked(self, *args: Any) -> None:
        logging.debug("Revert button pressed")

        if not (model := cast(Gtk.StringList, self.get_model())):
            return

        string_object = cast(
            Gtk.StringObject, self.settings.get_default_value(self.key)
        )
        if not string_object:
            return

        self.settings.reset(self.key)
        self.__set_selected(model, string_object.get_string())
        self.__set_default(cast(str, self.key), model)

    def __set_default(self, key: str, model: Gtk.StringList) -> None:
        default_string = self.settings.get_default_value(key).get_string()

        row_string = model.get_string(self.get_selected())

        self.is_default = row_string != default_string

    def __selected(self, *args: Any) -> None:
        if not (model := cast(Gtk.StringList, self.get_model())):
            return

        row_string = model.get_string(self.get_selected())

        self.__set_default(cast(str, self.key), model)
        self.settings.set_string(self.key, row_string)
        self.value_set(row_string)

    # TODO: Use `GtkStringList.find()` only when GTK 4.18 gets released
    def __set_selected(self, model: Gtk.StringList, string: str) -> None:
        try:
            self.set_selected(model.find(string))  # type: ignore [attr-defined]
        except AttributeError:
            for index in range(model.get_n_items()):
                if string == model.get_string(index):
                    self.set_selected(index)
                    break

    @RefineSettings.schema_id
    def schema_id(self, schema_id: str): ...

    @RefineSettings.key
    def key(self, key: str):
        try:
            if not self.schema.has_key(key):
                raise TypeError

            if not (model := cast(Gtk.StringList, self.get_model())):
                return

            if not (string := self.settings.get_string(key)):
                return

            self.__set_selected(model, string)

        except (IndexError, TypeError):
            self.is_valid_setting = False
            logging.warning(f"Key “{key}” does not exist. Skipping…")
            return

        self.__set_default(key, model)
        self.connect("notify::selected", self.__selected)

    @GObject.Property(type=str, default="")
    def data_subdirectory(self) -> str:
        """
        Get or set the data subdirectory. If the subdirectory is set, the
        `Gtk.StringList` is automatically set up for @self. If the
        `@self:lookup-directory` property is set, the list is filtered
        before creating the model.
        """

        return self._data_subdirectory

    @data_subdirectory.setter  # type: ignore [no-redef]
    def data_subdirectory(self, data_subdirectory: str) -> None:
        system_data_dirs_list = []
        for directory in get_system_data_dirs():
            found_directories = glob(
                f"{os.path.join(directory, data_subdirectory)}{os.sep}*"
            )
            system_data_dirs_list += found_directories

        data_dirs_list = system_data_dirs_list

        data_dirs_list += glob(
            f"{os.path.join(get_user_data_dir(), data_subdirectory)}{os.sep}/*"
        )

        if bool(self.lookup_directory):
            temp_dirs_list = []
            for directory in data_dirs_list:
                if os.path.isdir(os.path.join(directory, self.lookup_directory)):
                    temp_dirs_list.append(directory)
            data_dirs_list = temp_dirs_list

        data_dirs_list = filter(
            lambda directory: os.path.isdir(directory), data_dirs_list
        )
        data_dirs_list = list(set(map(os.path.basename, data_dirs_list)))
        data_dirs_model = Gtk.StringList.new(sorted(data_dirs_list, key=str.lower))
        self.set_model(data_dirs_model)

        self._data_subdirectory = data_subdirectory

    @GObject.Signal
    def inconsistent(self) -> None:
        """Log inconsistency signal."""

        logging.info(f"{self.__class__.__name__}::inconsistent signal emitted")

    def value_set(self, value: Any) -> None:
        """Log key-value pair."""

        logging.debug(f"Set key “{self.key}” to “{value}”")
